==> dispatchonvalue-0.9.0.tar.gz: dispatchonvalue-0.9.0/dispatchonvalue/dispatchonvalue.py <==
"""Provide dispatch on value for complex arbitrarily nested lists and
dictionaries.

You can use lambda to do expression matching and an 'any' token that is a
wildcard that ensures identical values can be matched. It is useful for getting
rid of complicated and difficult to read if...elif...elif... chains.

"""
import itertools


class AnyValue(object):
    """Allow wildcard item in DispatchOnValue."""
    def __init__(self, num):
        self.num = num

any_a = AnyValue(1)
any_b = AnyValue(2)
any_c = AnyValue(3)
any_d = AnyValue(4)
any_e = AnyValue(5)
any_f = AnyValue(6)
any_g = AnyValue(7)
any_h = AnyValue(8)
any_i = AnyValue(9)
any_j = AnyValue(10)
any_k = AnyValue(11)
any_l = AnyValue(12)
any_m = AnyValue(13)
any_n = AnyValue(14)
any_o = AnyValue(15)
any_p = AnyValue(16)
any_q = AnyValue(17)
any_r = AnyValue(18)
any_s = AnyValue(19)
any_t = AnyValue(20)
any_u = AnyValue(21)
any_v = AnyValue(22)
any_w = AnyValue(23)
any_x = AnyValue(24)
any_y = AnyValue(25)
any_z = AnyValue(26)


class DispatchOnValue(object):
    """Provide dispatch on value for complex arbitrarily nested lists and
    dictionaries."""

    def __init__(self):
        self.functions = []

    def add(self, pattern):
        """Decorator to add new dispatch functions."""
        copy_of_parent_class = self

        def wrap(f):
            def wrapped_f(*args):
                f(*args)

            copy_of_parent_class.functions.append((f, pattern))
            return wrapped_f

        return wrap

    def dispatch(self, stream, *args):
        """
        Dispatch to function held internally depending upon the value of stream.

        Matching on directories is partial. This means dictionaries will
        match if all the key/value pairs in the pattern are matched - any extra
        pairs will be ignored.

        """
        for t in self.functions:
            (matched, matched_stream) = self._match(
                stream, t[1], {'strict': False}
            )
            if matched:
                f = t[0]
                f(matched_stream, *args)
                return True

        return False

    def dispatch_strict(self, stream, *args):
        """
        Dispatch to function held internally depending upon the value of stream.

        Matching on directories is strict. This means dictionaries will
        match if they are exactly the same.

        """
        for t in self.functions:
            (matched, matched_stream) = self._match(
                stream, t[1], {'strict': True}
            )
            if matched:
                f = t[0]
                f(matched_stream, *args)
                return True

        return False

    def _match(self, stream, pattern, context):
        if isinstance(pattern, AnyValue):
            if pattern.num in context:
                return context[pattern.num] == stream, stream
            else:
                context[pattern.num] = stream
                return True, stream

        try:
            # First, is it callable?
            return pattern(stream), stream

        except TypeError:
            if not type(stream) == type(pattern):
                return False, []

            try:
                # OK, we'll assume it's a dictionary
                return self._compare_dictionaries(stream, pattern, context)

            except (AttributeError, TypeError):
                # Maybe a string or a list?
                try:
                    # I hate to add an isinstance() here but I can't see
                    # how to lose it
                    if isinstance(stream, basestring):
                        return self._compare_primitives(stream, pattern)
                    else:
                        return self._compare_lists(stream, pattern, context)

                except TypeError:
                    # Have to assume primitives
                    return self._compare_primitives(stream, pattern)

    @staticmethod
    def _compare_primitives(stream, pattern):
        if type(stream) == type(pattern) and stream == pattern:
            return True, stream
        else:
            return False, []

    def _compare_lists(self, stream, pattern, context):
        # We compare each item in the list. If they all match, then we have
        # a match.
        if not len(stream) == len(pattern):
            return False, []

        for s, p in itertools.izip(stream, pattern):
            (matched, matched_stream) = self._match(s, p, context)
            if not matched:
                return False, []

        return True, stream

    def _compare_dictionaries(self, stream, pattern, context):
        if 'strict' in context and context['strict']:
            if not len(stream) == len(pattern):
                return False, []

        for k, v in pattern.iteritems():
            if not k in stream:
                return False, []

            s = stream[k]
            (matched, matched_stream) = self._match(s, v, context)
            if not matched:
                return False, []

        return True, stream

==> dispatchonvalue-0.9.2.tar.gz: dispatchonvalue-0.9.2/dispatchonvalue/dispatchonvalue.py <==
"""Provide dispatch on value for complex arbitrarily nested lists and
dictionaries.

You can use lambda to do expression matching and an 'any' token that is a
wildcard that ensures identical values can be matched. It is useful for getting
rid of complicated and difficult to read if...elif...elif... chains.

"""
import itertools


class AnyValue(object):
    """Allow wildcard item in DispatchOnValue."""
    def __init__(self, num):
        self.num = num

any_a = AnyValue(1)
any_b = AnyValue(2)
any_c = AnyValue(3)
any_d = AnyValue(4)
any_e = AnyValue(5)
any_f = AnyValue(6)
any_g = AnyValue(7)
any_h = AnyValue(8)
any_i = AnyValue(9)
any_j = AnyValue(10)
any_k = AnyValue(11)
any_l = AnyValue(12)
any_m = AnyValue(13)
any_n = AnyValue(14)
any_o = AnyValue(15)
any_p = AnyValue(16)
any_q = AnyValue(17)
any_r = AnyValue(18)
any_s = AnyValue(19)
any_t = AnyValue(20)
any_u = AnyValue(21)
any_v = AnyValue(22)
any_w = AnyValue(23)
any_x = AnyValue(24)
any_y = AnyValue(25)
any_z = AnyValue(26)


class DispatchOnValue(object):
    """Provide dispatch on value for complex arbitrarily nested lists and
    dictionaries."""

    def __init__(self):
        self.functions = []

    def add(self, pattern):
        """Decorator to add new dispatch functions."""
        self_of_parent = self

        def wrap(f):
            def wrapped_f(*args):
                f(*args)

            self_of_parent.functions.append((f, pattern))
            return wrapped_f

        return wrap

    def dispatch(self, stream, *args):
        """
        Dispatch to function held internally depending upon the value of stream.

        Matching on directories is partial. This means dictionaries will
        match if all the key/value pairs in the pattern are matched - any extra
        pairs will be ignored.

        """
        for t in self.functions:
            (matched, matched_stream) = self._match(
                stream, t[1], {'strict': False}
            )
            if matched:
                f = t[0]
                f(matched_stream, *args)
                return True

        return False

    def dispatch_strict(self, stream, *args):
        """
        Dispatch to function held internally depending upon the value of stream.

        Matching on directories is strict. This means dictionaries will
        match if they are exactly the same.

        """
        for t in self.functions:
            (matched, matched_stream) = self._match(
                stream, t[1], {'strict': True}
            )
            if matched:
                f = t[0]
                f(matched_stream, *args)
                return True

        return False

    def _match(self, stream, pattern, context):
        if isinstance(pattern, AnyValue):
            if pattern.num in context:
                return context[pattern.num] == stream, stream
            else:
                context[pattern.num] = stream
                return True, stream

        try:
            # First, is it callable?
            return pattern(stream), stream

        except TypeError:
            if not type(stream) == type(pattern):
                return False, []

            try:
                # OK, we'll assume it's a dictionary
                return self._compare_dictionaries(stream, pattern, context)

            except (AttributeError, TypeError):
                # Maybe a string or a list?
                try:
                    # I hate to add an isinstance() here but I can't see
                    # how to lose it
                    if isinstance(stream, basestring):
                        return self._compare_primitives(stream, pattern)
                    else:
                        return self._compare_lists(stream, pattern, context)

                except TypeError:
                    # Have to assume primitives
                    return self._compare_primitives(stream, pattern)

    @staticmethod
    def _compare_primitives(stream, pattern):
        if type(stream) == type(pattern) and stream == pattern:
            return True, stream
        else:
            return False, []

    def _compare_lists(self, stream, pattern, context):
        # We compare each item in the list. If they all match, then we have
        # a match.
        if not len(stream) == len(pattern):
            return False, []

        for s, p in itertools.izip(stream, pattern):
            (matched, matched_stream) = self._match(s, p, context)
            if not matched:
                return False, []

        return True, stream

    def _compare_dictionaries(self, stream, pattern, context):
        if 'strict' in context and context['strict']:
            if not len(stream) == len(pattern):
                return False, []

        for k, v in pattern.iteritems():
            if not k in stream:
                return False, []

            s = stream[k]
            (matched, matched_stream) = self._match(s, v, context)
            if not matched:
                return False, []

        return True, stream
